import chimera, numpy, Tkinter as tk
from chimera import runCommand
from chimera.baseDialog import ModelessDialog
from chimera.baseDialog import ModalDialog
from numpy import array, zeros, dot, linalg
from Tkinter import Entry

from StructBio.SSEs.tubesForSSEs import TubesForSSEs
from StructBio.Scenographics.solids import Solids
from StructBio.Utilities.miscFunctions import residueKDhColor

#------------------------------------------------------------------------------
# InputPDBidDialog modal dialog class
class InputPDBidDialog(ModalDialog):
    title = "User Input"
    buttons = ('OK', 'Cancel')
    default = 'OK'
    oneshot = True
    help = False

    def __init__(self):
        ModalDialog.__init__(self)
        
    def fillInUI(self, parent):
        # Setting up a entry box: 
        tk.Label(parent, text =  " Type PDB ID:  ").grid(row = 0, column = 0,
                                                          sticky = tk.W)
        self.entryVar = tk.StringVar()   
        self.entryPDBid = Entry(parent, textvariable = self.entryVar, width = 8,
                                takefocus = 1, bg = "white")
        self.entryPDBid.grid(row = 1, pady = 10, padx = 4, column = 0, sticky = tk.E)
        self.entryPDBid.focus_set()

        self.checkedBoxHelices = tk.IntVar()
        cbA = tk.Checkbutton(parent, text = "Helices   ", variable = self.checkedBoxHelices,
                             command = None, indicatoron = 1)
        cbA.grid(row = 2, column = 0, pady = 6, sticky = tk.E)
        cbA.select()    # Starts this button with a check mark.

        self.checkedBoxStrands = tk.IntVar()
        cbA = tk.Checkbutton(parent, text = "Strands   ", variable = self.checkedBoxStrands,
                             command = None, indicatoron = 1)
        cbA.grid(row = 2, column = 1, pady = 6, sticky = tk.E)
        cbA.select()    # Starts this button with a check mark.

    def OK(self):
        self.resultPDBid = self.entryVar.get()
        self.doHelices = self.checkedBoxHelices.get()
        self.doStrands = self.checkedBoxStrands.get()
        ModalDialog.Cancel(self, value=None)

    def Cancel(self):
        self.resultPDBid = None 
        ModalDialog.Cancel(self, value=None)
        
#------------------------------------------------------------------------------
# ResNetDialog class (modeless dialog)
class HydrophobicMomentsDialog(ModelessDialog):
    buttons = ("Close")
    title = "Hydrophobic Moments"
    help = False

    def fillInUI(self, parent):
        #-------------------------------------------------------------------------
        # Set up radio buttons as specified by a list:
        rButtonText = tk.Label(parent, text =  " Choose type of display:  ")
        rButtonText.grid(row = 0, column = 0, pady = 4, columnspan = 2, sticky = tk.W)
        
        self.rBtnResult = tk.StringVar() # Use tk.IntVar() here for integer result values.
        radioBtn_L = [("Normals along axes ", "NA", 1, 0),
                      ("Normals at centroid", "NC", 1, 1),
                      ("Sum of normals     ", "SM", 2, 0),
                      ("Average of normals ", "AV", 2, 1)]
        
        self.rBtnResult.set("NA") # This button starts off selected.
        self.currentShow = "NA"
        for text, rslt, rw, cl in radioBtn_L:
            rBtn = tk.Radiobutton(parent, text = text, variable = self.rBtnResult,
                                  value = rslt, command = self.rBtnfunc)
            rBtn.grid(row = rw, column = cl, padx = 4, sticky = tk.W)

    #-------------------------------------------------------------------------
    # Callback function needed for radio buttons:
    def rBtnfunc(self):
        arrowSolids_D[self.currentShow].hide()
        self.currentShow = self.rBtnResult.get()
        arrowSolids_D[self.currentShow].show()

#------------------------------------------------------------------------------   

def residueCenter(r):
        retVal = zeros(3)
        aCount = 0
        for a in r.atoms:
            if a.name in ("CA", "C", "N", "O"): continue
            retVal += array(a.coord())
            aCount += 1
        return retVal/aCount

#------------------------------------------------------------------------------
# Get color based on residue hydrophobicity
def interpolateKDhColor(hyd): 
    white = array((1.0, 1.0, 1.0))              # (255, 255, 255)
    dodgerBlue = array((0.118, 0.565, 1.0 ))    # ( 30, 144, 255)
    orangeRed = array((1.0, 0.271, 0.0))        # (255, 69, 0)
    
    if hyd < 0.0:
        return tuple(white + (white - dodgerBlue)*hyd/4.5)
    else:
        return tuple(white + (orangeRed - white)*hyd/4.5)
#==============================================================================
# Mainline
#==============================================================================
runCommand("set bg_color white")

userDialog = InputPDBidDialog()
userDialog.run(chimera.tkgui.app)
if userDialog.resultPDBid:
    prot = chimera.openModels.open(userDialog.resultPDBid, type = "PDB")[0]
    doHelices = userDialog.doHelices
    doStrands = userDialog.doStrands

    axes_L = []
    if doHelices:
        tubesHelices = TubesForSSEs(prot, tolerance = 3.5, minLength = 3,
                                    helixTubeRadius = .5, strands = False)
        axes_L.extend(tubesHelices.axes.inertialAxes_L)
    if doStrands:
        tubesStrands = TubesForSSEs(prot, tolerance = 3.5, minLength = 2,
                                    strandTubeRadius = .5, helices = False)
        axes_L.extend(tubesStrands.axes.inertialAxes_L)    

    arrowSolids_D = {}  # GLOBAL variable
    arrowSolids_D["NA"] = Solids()    # Entry "NA": Normal arrows spread out along Axis
    arrowSolids_D["NC"] = Solids()    # Entry "NC": Normal arrows projected to Centroid
    arrowSolids_D["SM"] = Solids()    # Entry "SM": SuM of normals (hydrophilic & hydrophobic)
    arrowSolids_D["AV"] = Solids()    # Entry "AV": AVerage of normals.

    for inertialAxisObj in axes_L:
        arrowSumHydrophobic = zeros(3); arrowCountHydrophobic = 0.0; sumHydrophobic = 0.0
        arrowSumHydrophilic = zeros(3); arrowCountHydrophilic = 0.0; sumHydrophilic = 0.0
        # Work through the alpha carbons used to define each inertialAxisObj:
        for atom in inertialAxisObj.atoms_L:    
            if atom.residue.type == "GLY": continue

            centroid = inertialAxisObj.centroid
            primaryAxis = inertialAxisObj.primaryAxis

            # Compute residue center:
            rc = residueCenter(atom.residue)
            # Compute residueCenter relative to centroid:
            rcrc =  rc - centroid        
            # Compute projection of rc onto axis:
            rcProj = primaryAxis*dot(primaryAxis, rcrc) + centroid

            arrow = rc - rcProj     # Arrow from rcProj to rc (relative to PDB origin)

            #Change length of arrow to correspond to hydrophobicity:
            arrowHyd = arrow*abs(atom.residue.kdHydrophobicity)/linalg.norm(arrow)
            rcHyd = rcProj + arrowHyd

            arrowSolids_D["NA"].addArrow(rcProj, rcHyd, 0.2, residueKDhColor(atom.residue))

            arrowEnd = rcHyd - rcProj + centroid
            arrowSolids_D["NC"].addArrow(centroid, arrowEnd,
                                         0.2, residueKDhColor(atom.residue))
            if atom.residue.kdHydrophobicity > 0.0:
                arrowSumHydrophobic += arrowHyd
                arrowCountHydrophobic += 1.
                sumHydrophobic += atom.residue.kdHydrophobicity
            else:
                arrowSumHydrophilic += arrowHyd
                arrowCountHydrophilic += 1.
                sumHydrophilic += atom.residue.kdHydrophobicity

        if arrowCountHydrophobic > 0:
            arrowSolids_D["SM"].addArrow(centroid, arrowSumHydrophobic + centroid,
                                         0.2, interpolateKDhColor(sumHydrophobic/arrowCountHydrophobic))
            arrowSolids_D["AV"].addArrow(centroid, arrowSumHydrophobic/arrowCountHydrophobic + centroid,
                                         0.2, interpolateKDhColor(sumHydrophobic/arrowCountHydrophobic))

        if arrowCountHydrophilic > 0:
            arrowSolids_D["SM"].addArrow(centroid, arrowSumHydrophilic + centroid,
                                         0.2, interpolateKDhColor(sumHydrophilic/arrowCountHydrophilic))
            arrowSolids_D["AV"].addArrow(centroid, arrowSumHydrophilic/arrowCountHydrophilic + centroid,
                                         0.2, interpolateKDhColor(sumHydrophilic/arrowCountHydrophilic))       

    arrowSolids_D["NA"].display(); arrowSolids_D["NC"].display()
    arrowSolids_D["SM"].display(); arrowSolids_D["AV"].display()
    arrowSolids_D["NC"].hide(); arrowSolids_D["SM"].hide(); arrowSolids_D["AV"].hide();

    HydrophobicMomentsDialog()















