import chimera, numpy
from chimera import runCommand
from numpy import array

from StructBio.Scenographics.surfaces import Surfaces 
from StructBio.HBonds import Dehydrons

#==============================================================================
# HBondPlatesWrapColored CLASS
#==============================================================================

class HBondPlatesWrapColored(object):
    def __init__(self, proteinMolecule, sphereRadius = 6.5, wrapThreshold = 19):
        
        self.prot = proteinMolecule
        self.sphereRadius = sphereRadius
        self.wrapThreshold = wrapThreshold

        plateSubdivisions = 64
        
        self.__pd = Surfaces(modelName = "H-Bond Plates")

        dehydrons = Dehydrons(prot, sphereRadius = self.sphereRadius, wrapThreshold = self.wrapThreshold)
        
        for dn_acPair in dehydrons.hBonds_L:
            donorA = dn_acPair[0]
            acptrA = dn_acPair[1]

            # Be sure flanking residues exist (we might be at the end of a chain):
            if not self.prot.residueBefore(donorA.residue): continue
            if not self.prot.residueBefore(donorA.residue).findAtom('CA'): continue
            if not self.prot.residueAfter(acptrA.residue): continue
            if not self.prot.residueAfter(acptrA.residue).findAtom('CA'): continue
            # Flanking residues must be in same chain:
            if not self.prot.residueAfter(acptrA.residue).id.chainId == acptrA.residue.id.chainId: continue
            if not self.prot.residueBefore(donorA.residue).id.chainId == donorA.residue.id.chainId: continue
            
            coordsCaInDonorRes = array(donorA.residue.findAtom('CA').coord())
            coordsCaInAcptrRes = array(acptrA.residue.findAtom('CA').coord())
            coordsCaBeforeDonorRes = array(self.prot.residueBefore(donorA.residue).findAtom('CA').coord())
            coordsCaAfterAcptrRes = array(self.prot.residueAfter(acptrA.residue).findAtom('CA').coord())

            parallel = numpy.dot(coordsCaInDonorRes - coordsCaBeforeDonorRes, coordsCaAfterAcptrRes - coordsCaInAcptrRes)
            if parallel > 0.0:
                v = [coordsCaBeforeDonorRes, coordsCaInDonorRes,
                     coordsCaAfterAcptrRes,coordsCaInAcptrRes]
            else:
                v = [coordsCaBeforeDonorRes, coordsCaInDonorRes,
                     coordsCaInAcptrRes, coordsCaAfterAcptrRes]
                
            if (donorA, acptrA) in dehydrons.wrapper_D.keys():
                wC = len(dehydrons.wrapper_D[(donorA, acptrA)])  # Wrapper count
                # Normalize wrap count to a value in range [-1, 1].
                normalizedWC = (min(max(wC, self.wrapThreshold - 10), self.wrapThreshold + 10) - self.wrapThreshold)/10.0
                greenEntry = .5 - .5*normalizedWC
                redBlueEntry = .5 +  .5*normalizedWC
                plateColor = (redBlueEntry, greenEntry, redBlueEntry)

                p_L = []
                q_L = []
                for i in range(plateSubdivisions + 1):
                    p_L.append(v[0] + (v[1] - v[0])*float(i)/plateSubdivisions)
                    q_L.append(v[3] + (v[2] - v[3])*float(i)/plateSubdivisions)
                for i in range(plateSubdivisions):
                    self.__pd.addPolygon([q_L[i], p_L[i], p_L[i+1], q_L[i+1]], plateColor, 1.0)
                #self.__pd.addPolygon(v, plateColor, 1.0)
                    

        self.__pd.display()
        
        runCommand("set bg_color white")
        runCommand("~ribbon")
        runCommand("~select")
        runCommand("chain @ca")
        runCommand("repr stick")
        runCommand("display sel")
        runCommand("~select")     

#==============================================================================
# Mainline
#==============================================================================
# try 3S75 to see a very extensive beta sheet.
protID = raw_input("Type in PDB ID:  ")

prot = chimera.openModels.open(protID, type="PDB")[0]

hbPlatesWrapColored = HBondPlatesWrapColored(prot)






                 
